/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2019                               **
**                                                                        **
**                             MODULE trace                               **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2019.01.19                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/


#include "trace.h"
#include "uart0.h"
#include "strings.h"
#include "frameutil.h"


static UCHAR TMessage01 [ ]      = "Ramka ETH, DEST=" ;
static UCHAR TMessage02 [ ]      = " SRC=" ;
static UCHAR TMessage03 [ ]      = "\r\nPakiet ARP" ;
static UCHAR TMessage04 [ ]      = "\r\nPakiet IP" ;
static UCHAR TMessage05 [ ]      = "\r\nPakiet ???" ;
static UCHAR TMessage06 [ ]      = "/ICMP" ;
static UCHAR TMessage07 [ ]      = "/TCP" ;
static UCHAR TMessage08 [ ]      = "/UDP" ;
static UCHAR TMessage09 [ ]      = "/???" ;

void TraceFrame ( PtrOSFrameRecT Frame )
{
  EtherPackedHeaderRecT EtherHeader ;
  IPHeaderRecT IPHeader ;
  /*-------------------------------------------------------------------------*/
  GetEthHeader ( & EtherHeader , Frame ) ;
  UART0SendString ( TMessage01 ) ;
  TraceMAC ( EtherHeader . DestinMACAddress ) ;
  UART0SendString ( TMessage02 ) ;
  TraceMAC ( EtherHeader . SourceMACAddress ) ;
  switch ( EtherHeader . LengthOrType )
  {
    case FRAME_ARP :
      UART0SendString ( TMessage03 ) ;
      break ;
    case FRAME_IP :
      UART0SendString ( TMessage04 ) ;
      SetIPHeaderOrgin ( Frame ) ; 
      GetIPHeader ( & IPHeader , Frame ) ;
      switch ( IPHeader . Protocol )
      {
        case Prot_ICMP                :
          UART0SendString ( TMessage06 ) ;
          break ;
        case Prot_TCP                 :
          UART0SendString ( TMessage07 ) ;
          break ;
        case Prot_UDP                 :
          UART0SendString ( TMessage08 ) ;
          break ;
        default :
          UART0SendString ( TMessage09 ) ;
       } /* switch */ ;
      break ;
    default :
      UART0SendString ( TMessage05 ) ;
  } /* switch */ ;
  UART0Send ( 0x0D ) ;
  UART0Send ( 0x0A ) ;
} /* TraceFrame */

static UCHAR IPMessage10 [ ]          = "\r\nNaglowek IP:" ;
static UCHAR IPMessage11 [ ]          = "\r\nVERS=" ;
static UCHAR IPMessage12 [ ]          = " HLEN=" ;
static UCHAR IPMessage13 [ ]          = " TOS=" ;
static UCHAR IPMessage14 [ ]          = " TOTAL=" ;
static UCHAR IPMessage15 [ ]          = "\r\nIDENT=" ;
static UCHAR IPMessage16 [ ]          = " FLAGS=" ;
static UCHAR IPMessage17 [ ]          = " FRAGOFFS=" ;
static UCHAR IPMessage18 [ ]          = "\r\nTTL=" ;
static UCHAR IPMessage19 [ ]          = " PROT=" ;
static UCHAR IPMessage20 [ ]          = " HCRC=" ;
static UCHAR IPMessage21 [ ]          = "\r\nSRC=" ;
static UCHAR IPMessage22 [ ]          = "\r\nDST=" ;

void TraceIPHeader ( IPHeaderRecT * IPHeader )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( IPMessage10 ) ;
  UART0SendString ( IPMessage11 ) ;
  TraceUCHAR ( IPHeader -> Vers ) ;
  UART0SendString ( IPMessage12 ) ;
  TraceUCHAR ( IPHeader -> HLen ) ;
  UART0SendString ( IPMessage13 ) ;
  TraceUCHAR ( IPHeader -> ServiceType ) ;
  UART0SendString ( IPMessage14 ) ;
  TraceUSHORT ( IPHeader -> TotalLength ) ;
  UART0SendString ( IPMessage15 ) ;
  TraceUSHORT ( IPHeader -> Identification ) ;
  UART0SendString ( IPMessage16 ) ;
  TraceUCHAR ( IPHeader -> Flags ) ;
  UART0SendString ( IPMessage17 ) ;
  TraceUSHORT ( IPHeader -> FragmentOffs ) ;
  UART0SendString ( IPMessage18 ) ;
  TraceUCHAR ( IPHeader -> TTL ) ;
  UART0SendString ( IPMessage19 ) ;
  TraceUSHORT ( IPHeader -> Protocol ) ;
  UART0SendString ( IPMessage20 ) ;
  TraceUSHORT ( IPHeader -> HeaderChecksum ) ;
  UART0SendString ( IPMessage21 ) ;
  TraceIP ( IPHeader -> SourceIPAddress ) ;
  UART0SendString ( IPMessage22 ) ;
  TraceIP ( IPHeader -> DestinationIPAddress ) ;
  UART0Send ( 0x0D ) ;
  UART0Send ( 0x0A ) ;
} /* TraceIPHeader */


UCHAR Message01 [ ]              = "\r\nOdebrano ramke do MAC=" ;
UCHAR Message02 [ ]              = " od MAC=" ;
UCHAR Message03 [ ]              = ", jest to pakiet ARP\r\n" ;
UCHAR Message04 [ ]              = ", jest to pakiet IP\r\n" ;
UCHAR Message05 [ ]              = ", jest to pakiet ???\r\n" ;

void TraceETHHeader ( EtherPackedHeaderRecT * EtherPackedHeader )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Message01 ) ;
  TraceMAC ( EtherPackedHeader -> DestinMACAddress ) ;
  UART0SendString ( Message02 ) ;
  TraceMAC ( EtherPackedHeader -> SourceMACAddress ) ;
  switch ( EtherPackedHeader -> LengthOrType )
  {
    case FRAME_ARP    :
      UART0SendString ( Message03 ) ;
      break ;
    case FRAME_IP     :
      UART0SendString ( Message04 ) ;
      break ;
    default :
      UART0SendString ( Message05 ) ;
  } /* switch */ ;
} /* TraceETHHeader */


void TraceMAC ( UCHAR * MACAddr )
{  
  UCHAR NumberStr [ 32 ] ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < 5 ; Loop ++ )
  {
    WordToHexStr ( NumberStr , 2 , '0' ,  ( USHORT ) ( * MACAddr ) ) ;
    MACAddr ++ ;
    UART0SendString ( NumberStr ) ;
    UART0Send ( '-' ) ;
  } /* for */ ;  
  WordToHexStr ( NumberStr , 2 , '0' ,  ( USHORT ) ( * MACAddr ) ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceMAC */


void TraceIP ( ULONG IPAddress )
{
  UCHAR NumberStr [ 24 ] ;
  /*-------------------------------------------------------------------------*/
  IPToString ( NumberStr , IPAddress ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceIP */


void TraceUCHAR ( UCHAR Data )
{
  UCHAR NumberStr [ 8 ] ;
  /*-------------------------------------------------------------------------*/
  WordToHexStr ( NumberStr , 2 , '0' , ( USHORT ) Data ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceUCHAR */
  

void TraceUSHORT ( USHORT Data )
{
  UCHAR NumberStr [ 8 ] ;
  /*-------------------------------------------------------------------------*/
  WordToHexStr ( NumberStr , 4 , '0' , Data ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceUSHORT */


void TraceULONG ( ULONG Data )
{
  UCHAR NumberStr [ 12 ] ;
  /*-------------------------------------------------------------------------*/
  LongToHexStr ( NumberStr , 8 , '0' , Data ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceULONG */

